import type { ILogger } from "../../Parser/ILogger";
import { BpmxObject } from "./bpmxObject";
/**
 * BpmxReader is a static class that parses BPMX data
 */
export declare class BpmxReader {
    private constructor();
    /**
     * Parses BPMX data asynchronously
     * @param data Arraybuffer of BPMX data
     * @param logger Logger
     * @returns BPMX data
     * @throws {RangeError} If the parse fails
     * @throws {LoadFileError} If the BPMX version is not supported
     */
    static ParseAsync(data: ArrayBufferLike, logger?: ILogger): Promise<BpmxObject>;
    private static _ParseHeader;
    private static _ParseGeometriesAsync;
    private static _ParseImagesAsync;
    private static _ParseTexturesAsync;
    private static _ParseMaterials;
    private static _ParseBones;
    private static _ParseMorphs;
    private static _ParseDisplayFrames;
    private static _ParseRigidBodies;
    private static _ParseJoints;
}
