import type { ILogger } from "./ILogger";
import { PmxObject } from "./pmxObject";
/**
 * PmdReader is a static class that parses PMD data
 */
export declare class PmdReader {
    private constructor();
    /**
     * Parses PMD data asynchronously
     * @param data Arraybuffer of PMD data
     * @param logger Logger
     * @returns PMD data as a PmxObject
     * @throws {Error} If the parse fails
     */
    static ParseAsync(data: ArrayBufferLike, logger?: ILogger): Promise<PmxObject>;
    private static _ParseHeader;
    private static _ParseVerticesAsync;
    private static _ParseIndices;
    private static _ParseMaterials;
    private static _ParseBones;
    private static _ParseIks;
    private static _ParseMorphs;
    private static _ParseDisplayFrames;
    private static _ParseEnglishNames;
    private static _ParseToonTextures;
    private static _PathNormalize;
    private static _ConvertMaterials;
    /**
     * from pmx editor IK制限角.txt
     * format: minX, maxX, minY, maxY, minZ, maxZ
     *
     * 左ひざ,-180.0,-0.5,0.0,0.0,0.0,0.0
     * 右ひざ,-180.0,-0.5,0.0,0.0,0.0,0.0
     */
    private static readonly _IkAngleLimitTable;
    private static _ConvertBones;
    private static _ParseRigidBodies;
    private static _NormalizeRigidBodyPositions;
    private static _ParseJoints;
}
