import { type ILogger } from "./ILogger";
import { PmxObject } from "./pmxObject";
/**
 * PmxReader is a static class that parses PMX data
 */
export declare class PmxReader {
    private constructor();
    /**
     * Parses PMX data asynchronously
     * @param data Arraybuffer of PMX data
     * @param logger Logger
     * @returns PMX data
     * @throws {Error} If the parse fails
     */
    static ParseAsync(data: ArrayBufferLike, logger?: ILogger): Promise<PmxObject>;
    private static _ParseHeader;
    private static _ParseVerticesAsync;
    private static _ParseIndices;
    private static _ParseTextures;
    private static _ParseMaterials;
    private static _ParseBones;
    private static _ParseMorphs;
    private static _ParseDisplayFrames;
    private static _ParseRigidBodies;
    private static _ParseJoints;
    private static _ParseSoftBodies;
}
