import type { AbstractEngine } from "@babylonjs/core/Engines/abstractEngine";
/**
 * Sdef injector
 *
 * This class is used to inject SDEF code into the vertex shader
 */
export declare class SdefInjector {
    /**
     * Override engine create effect
     *
     * Inject the code so that all GLSL fragment shaders support Spherical Defromation(SDEF)
     *
     * If you want shadow map or postprocessing shaders to support SDEF call this method before creating the shadow map or postprocess render pipeline
     * @param engine Engine
     */
    static OverrideEngineCreateEffect(engine: AbstractEngine): void;
    /**
     * Returns the result of putting the SDEF code in the shader
     *
     * You can use this method if you want to manually inject shader
     *
     * This method is usually not used directly
     * @param shaderType Shader type
     * @param code Shader code
     * @returns Shader code that SDEF is injected
     */
    static ProcessSdefCode(shaderType: string, code: string): string;
}
